@@ -1,9 +1,7 @@
+use crate::command::trove::CommandTrove;
+use crate::gui::prompts::{prompt_input, prompt_input_validate};
 use serde::{Deserialize, Serialize};
 
-use dialoguer::{theme::ColorfulTheme, Input};
-
-use super::trove::CommandTrove;
-
 pub trait Parsable {
     fn parse_arguments(matches: &clap::ArgMatches) -> Self;
 }
@@ -20,13 +18,14 @@ pub struct HoardCommand {
 impl HoardCommand {
     pub fn default() -> Self {
         Self {
-            name: String::from(""),
-            namespace: String::from(""),
+            name: "".to_string(),
+            namespace: "".to_string(),
             tags: None,
-            command: String::from(""),
+            command: "".to_string(),
             description: None,
         }
     }
+
     #[allow(dead_code)]
     pub fn is_complete(&self) -> bool {
         if self.name.is_empty()
@@ -43,24 +42,31 @@ impl HoardCommand {
     pub fn tags_as_string(&self) -> String {
         self.tags
             .as_ref()
-            .unwrap_or(&vec![String::from("")])
+            .unwrap_or(&vec!["".to_string()])
             .join(",")
     }
 
-    pub fn with_command_string_input(self, default_value: Option<String>) -> Self {
-        let command_string: String;
-        if let Some(val) = default_value {
-            command_string = Input::with_theme(&ColorfulTheme::default())
-                .default(val)
-                .with_prompt("Command to hoard")
-                .interact_text()
-                .unwrap();
-        } else {
-            command_string = Input::with_theme(&ColorfulTheme::default())
-                .with_prompt("Command to hoard")
-                .interact_text()
-                .unwrap();
+    #[allow(dead_code)]
+    pub fn with_command_raw(self, command_string: &str) -> Self {
+        Self {
+            name: self.name,
+            namespace: self.namespace,
+            tags: self.tags,
+            command: command_string.to_string(),
+            description: self.description,
         }
+    }
+
+    pub fn with_command_string_input(
+        self,
+        default_value: Option<String>,
+        parameter_token: &str,
+    ) -> Self {
+        let base_prompt = format!(
+            "Command to hoard ( Mark unknown parameters with {} )\n",
+            parameter_token
+        );
+        let command_string: String = prompt_input(&base_prompt, false, default_value);
         Self {
             name: self.name,
             namespace: self.namespace,
@@ -88,54 +94,24 @@ impl HoardCommand {
     }
 
     pub fn with_tags_input(self, default_value: Option<String>) -> Self {
-        let tags: String;
-        if let Some(val) = default_value {
-            tags = Input::with_theme(&ColorfulTheme::default())
-                .default(val)
-                .with_prompt("Give your command tags ( comma separated )")
-                .validate_with({
-                    move |input: &String| -> Result<(), &str> {
-                        if input.contains(' ') {
-                            Err("Tags cant contain whitespaces")
-                        } else {
-                            Ok(())
-                        }
-                    }
-                })
-                .interact_text()
-                .unwrap();
-        } else {
-            tags = Input::with_theme(&ColorfulTheme::default())
-                .with_prompt("Give your command tags ( comma separated )")
-                .validate_with({
-                    move |input: &String| -> Result<(), &str> {
-                        if input.contains(' ') {
-                            Err("Tags cant contain whitespaces")
-                        } else {
-                            Ok(())
-                        }
-                    }
-                })
-                .interact_text()
-                .unwrap();
-        }
+        let tag_validator = move |input: &String| -> Result<(), String> {
+            if input.contains(' ') {
+                Err("Tags cant contain whitespaces".to_string())
+            } else {
+                Ok(())
+            }
+        };
+        let tags: String = prompt_input_validate(
+            "Give your command some optional tags ( comma seperated )",
+            true,
+            default_value,
+            Some(tag_validator),
+        );
         self.with_tags_raw(&tags)
     }
 
     pub fn with_namespace_input(self, default_namespace: Option<String>) -> Self {
-        let namespace: String;
-        if let Some(val) = default_namespace {
-            namespace = Input::with_theme(&ColorfulTheme::default())
-                .default(val)
-                .with_prompt("Namespace of the command")
-                .interact_text()
-                .unwrap();
-        } else {
-            namespace = Input::with_theme(&ColorfulTheme::default())
-                .with_prompt("Namespace of the command")
-                .interact_text()
-                .unwrap();
-        }
+        let namespace: String = prompt_input("Namespace of the command", false, default_namespace);
         Self {
             name: self.name,
             namespace,
@@ -149,54 +125,27 @@ impl HoardCommand {
         self,
         default_value: Option<String>,
         trove: &CommandTrove,
-        prompt_string: String,
+        prompt_string: &str,
     ) -> Self {
-        let command_names = trove.commands.clone();
         let namespace = self.namespace.clone();
-
-        let name: String;
-        if let Some(val) = default_value {
-            name = Input::with_theme(&ColorfulTheme::default())
-                .default(val)
-                .with_prompt(prompt_string)
-                .validate_with({
-                    move |input: &String| -> Result<(), &str> {
-                        if input.contains(' ') {
-                            Err("The name cant contain whitespaces")
-                        } else if command_names
-                            .iter()
-                            .filter(|x| x.namespace == namespace)
-                            .any(|x| x.name == input.to_string())
-                        {
-                            Err("A command with same name exists in the this namespace. Input a different name")
-                        } else {
-                            Ok(())
-                        }
-                    }
-                })
-                .interact_text()
-                .unwrap();
-        } else {
-            name = Input::with_theme(&ColorfulTheme::default())
-                .with_prompt(prompt_string)
-                .validate_with({
-                    move |input: &String| -> Result<(), &str> {
-                        if input.contains(' ') {
-                            Err("The name cant contain whitespaces")
-                        } else if command_names
-                            .iter()
-                            .filter(|x| x.namespace == namespace)
-                            .any(|x| x.name == input.to_string())
-                        {
-                            Err("A command with same name exists in the this namespace. Input a different name")
-                        } else {
-                            Ok(())
-                        }
-                    }
-                })
-                .interact_text()
-                .unwrap();
-        }
+        let command_names = trove.commands.clone();
+        let validator = move |input: &String| -> Result<(), String> {
+            if input.contains(' ') {
+                Err("The name cant contain whitespaces".to_string())
+            } else if command_names
+                .iter()
+                .filter(|x| x.namespace == namespace)
+                .any(|x| x.name == *input)
+            {
+                Err(
+                    "A command with same name exists in the this namespace. Input a different name"
+                        .to_string(),
+                )
+            } else {
+                Ok(())
+            }
+        };
+        let name = prompt_input_validate(prompt_string, false, default_value, Some(validator));
         Self {
             name,
             namespace: self.namespace,
@@ -207,7 +156,7 @@ impl HoardCommand {
     }
 
     pub fn with_name_input(self, default_value: Option<String>, trove: &CommandTrove) -> Self {
-        self.with_name_input_prompt(default_value, trove, "Name your command".to_string())
+        self.with_name_input_prompt(default_value, trove, "Name your command")
     }
 
     pub fn with_alt_name_input(self, default_value: Option<String>, trove: &CommandTrove) -> Self {
@@ -217,7 +166,7 @@ impl HoardCommand {
         self.with_name_input_prompt(
             default_value,
             trove,
-            format!(
+            &format!(
                 "A command with same name already exists in the namespace '{}'. Enter an alternate name for '{}' with command `{}`",
                 namespace,
                 name,
@@ -227,19 +176,8 @@ impl HoardCommand {
     }
 
     pub fn with_description_input(self, default_value: Option<String>) -> Self {
-        let description_string: String;
-        if let Some(val) = default_value {
-            description_string = Input::with_theme(&ColorfulTheme::default())
-                .default(val)
-                .with_prompt("Describe what the command does")
-                .interact_text()
-                .unwrap();
-        } else {
-            description_string = Input::with_theme(&ColorfulTheme::default())
-                .with_prompt("Describe what the command does")
-                .interact_text()
-                .unwrap();
-        }
+        let description_string: String =
+            prompt_input("Describe what the command does", false, default_value);
         Self {
             name: self.name,
             namespace: self.namespace,
@@ -255,12 +193,10 @@ impl Parsable for HoardCommand {
         let mut new_command = Self::default();
 
         if let Some(n) = matches.value_of("name") {
-            // TODO: some name validation for when we have it
             new_command.name = n.to_string();
         }
         // Defaults to 'default' namespace
         if let Some(ns) = matches.value_of("namespace") {
-            // TODO: some validation for when we have it
             new_command.namespace = ns.to_string();
         }
         // "$ hoard test -t" was run
@@ -273,13 +209,94 @@ impl Parsable for HoardCommand {
             );
         }
         if let Some(c) = matches.value_of("command") {
-            // TODO: some validation for when we have it
             new_command.command = c.to_string();
         }
         new_command
     }
 }
 
+pub trait Parameterized {
+    // Check if parameter pointers are present
+    fn is_parameterized(&self, token: &str) -> bool;
+    // Count number of parameter pointers
+    fn get_parameter_count(&self, token: &str) -> usize;
+    fn split(&self, token: &str) -> Vec<String>;
+    // Get parameterized Stringlike subject including parameter token
+    // For example, given subject with parameter token '#1':
+    // 'This is a #1 with one parameter token'
+    // `get_split_subject("#")` returns
+    // Vec['This is a ', '#', ' with one parameter token']
+    fn get_split_subject(&self, token: &str) -> Vec<String>;
+    // Replaces parameter tokens with content from `parameters`,
+    // consuming entries one by one until `parameters` is empty.
+    fn replace_parameters(self, token: &str, parameters: &[String]) -> HoardCommand;
+
+    fn with_input_parameters(self, token: &str) -> HoardCommand;
+}
+
+impl Parameterized for HoardCommand {
+    fn is_parameterized(&self, token: &str) -> bool {
+        self.command.contains(token)
+    }
+    fn get_parameter_count(&self, token: &str) -> usize {
+        self.command.matches(token).count()
+    }
+    fn split(&self, token: &str) -> Vec<String> {
+        self.command.split(token).map(ToString::to_string).collect()
+    }
+    fn get_split_subject(&self, token: &str) -> Vec<String> {
+        let split = self.split(token);
+        let mut collected: Vec<String> = Vec::new();
+        for s in split {
+            collected.push(s.clone());
+            collected.push(token.to_string());
+        }
+        collected
+    }
+    fn replace_parameters(self, token: &str, parameters: &[String]) -> HoardCommand {
+        let mut parameter_iter = parameters.iter();
+        let split = self.split(token);
+        let mut collected: Vec<String> = Vec::new();
+        for s in split {
+            collected.push(s.clone());
+            collected.push(parameter_iter.next().unwrap_or(&token.to_string()).clone());
+        }
+        collected.pop();
+        Self {
+            name: self.name,
+            namespace: self.namespace,
+            tags: self.tags,
+            command: collected.concat(),
+            description: self.description,
+        }
+    }
+
+    fn with_input_parameters(self, token: &str) -> HoardCommand {
+        let parameter_count = self.get_parameter_count(token);
+        if parameter_count == 0 {
+            return self;
+        }
+        let mut command_state = self.command.clone();
+        for i in 0..parameter_count {
+            let prompt_dialoge = format!(
+                "Enter parameter({}) nr {} \n~> {}\n",
+                token,
+                (i + 1),
+                command_state
+            );
+            let parameter = prompt_input(&prompt_dialoge, false, None);
+            command_state = command_state.replacen(token, &parameter, 1);
+        }
+        Self {
+            name: self.name,
+            namespace: self.namespace,
+            tags: self.tags,
+            command: command_state,
+            description: self.description,
+        }
+    }
+}
+
 #[cfg(test)]
 mod test_commands {
     use super::*;
@@ -308,7 +325,7 @@ mod test_commands {
     #[test]
     fn parse_single_tag() {
         let command = HoardCommand::default().with_tags_raw("foo");
-        let expected = Some(vec![String::from("foo")]);
+        let expected = Some(vec!["foo".to_string()]);
         assert_eq!(expected, command.tags);
     }
 
@@ -322,14 +339,65 @@ mod test_commands {
     #[test]
     fn parse_multiple_tags() {
         let command = HoardCommand::default().with_tags_raw("foo,bar");
-        let expected = Some(vec![String::from("foo"), String::from("bar")]);
+        let expected = Some(vec!["foo".to_string(), "bar".to_string()]);
         assert_eq!(expected, command.tags);
     }
 
     #[test]
     fn parse_whitespace_in_tags() {
         let command = HoardCommand::default().with_tags_raw("foo, bar");
-        let expected = Some(vec![String::from("foo"), String::from("bar")]);
+        let expected = Some(vec!["foo".to_string(), "bar".to_string()]);
         assert_eq!(expected, command.tags);
     }
 }
+
+#[cfg(test)]
+mod test_parameterized {
+    use super::*;
+
+    fn command_struct(command: &str) -> HoardCommand {
+        HoardCommand::default().with_command_raw(command)
+    }
+
+    #[test]
+    fn test_split() {
+        let token = "#".to_string();
+        let c: HoardCommand = command_struct("test # test");
+        let expected = vec!["test ".to_string(), " test".to_string()];
+        assert_eq!(expected, c.split(&token));
+    }
+
+    #[test]
+    fn test_split_empty() {
+        let token = "#".to_string();
+        let c: HoardCommand = command_struct("test  test");
+        let expected = vec!["test  test".to_string()];
+        assert_eq!(expected, c.split(&token));
+    }
+
+    #[test]
+    fn test_split_multiple() {
+        let token = "#".to_string();
+        let c: HoardCommand = command_struct("test # test #");
+        let expected = vec!["test ".to_string(), " test ".to_string(), "".to_string()];
+        assert_eq!(expected, c.split(&token));
+    }
+
+    #[test]
+    fn test_replace_parameters() {
+        let token = "#".to_string();
+        let c: HoardCommand = command_struct("test # bar");
+        let to_replace = vec!["foo".to_string()];
+        let expected = "test foo bar".to_string();
+        assert_eq!(expected, c.replace_parameters(&token, &to_replace).command);
+    }
+
+    #[test]
+    fn test_replace_last_parameters() {
+        let token = "#".to_string();
+        let c: HoardCommand = command_struct("test foo #");
+        let to_replace = vec!["bar".to_string()];
+        let expected = "test foo bar".to_string();
+        assert_eq!(expected, c.replace_parameters(&token, &to_replace).command);
+    }
+}
