@@ -3,11 +3,13 @@ use log::info;
 use prettytable::{color, Attr, Cell, Row, Table};
 use serde::{Deserialize, Serialize};
 
+use std::collections::HashSet;
 use std::{fs, path::Path, path::PathBuf};
 
-use super::hoard_command::HoardCommand;
+use crate::command::hoard_command::{HoardCommand, Parameterized};
+use crate::config::HoardConfig;
 
-const CARGO_VERION: &str = env!("CARGO_PKG_VERSION");
+const CARGO_VERSION: &str = env!("CARGO_PKG_VERSION");
 
 #[allow(clippy::module_name_repetitions)]
 #[derive(Debug, Serialize, Deserialize)]
@@ -19,7 +21,7 @@ pub struct CommandTrove {
 impl Default for CommandTrove {
     fn default() -> Self {
         Self {
-            version: String::from(CARGO_VERION),
+            version: CARGO_VERSION.to_string(),
             commands: Vec::new(),
         }
     }
@@ -77,7 +79,7 @@ impl CommandTrove {
 
     pub fn add_command(&mut self, new_command: HoardCommand) {
         let new_command = if self.check_name_collision(&new_command) {
-            new_command.with_alt_name_input(None, &self)
+            new_command.with_alt_name_input(None, self)
         } else {
             new_command
         };
@@ -102,10 +104,26 @@ impl CommandTrove {
         Ok(())
     }
 
-    pub fn pick_command(&self, name: &str) -> Result<HoardCommand> {
+    pub fn namespaces(&self) -> Vec<&str> {
+        let mut namespaces: Vec<_> = self
+            .commands
+            .iter()
+            .map(|command| command.namespace.as_str())
+            .collect::<HashSet<_>>()
+            .into_iter()
+            .collect();
+
+        namespaces.sort_unstable();
+        namespaces
+    }
+
+    pub fn pick_command(&self, config: &HoardConfig, name: &str) -> Result<HoardCommand> {
         let filtered_command: Option<&HoardCommand> = self.commands.iter().find(|c| c.name == name);
         if let Some(command) = filtered_command {
-            Ok(command.clone())
+            let command = command
+                .clone()
+                .with_input_parameters(&config.parameter_token.clone().unwrap());
+            Ok(command)
         } else {
             return Err(anyhow!("No matching command found with name: {}", name));
         }
@@ -115,11 +133,11 @@ impl CommandTrove {
         self.commands.is_empty()
     }
 
-    pub fn merge_trove(&mut self, other: CommandTrove) {
+    pub fn merge_trove(&mut self, other: &Self) {
         other
             .commands
             .iter()
-            .for_each(|c| self.add_command(c.clone()))
+            .for_each(|c| self.add_command(c.clone()));
     }
 
     pub fn print_trove(&self) {
@@ -141,7 +159,7 @@ impl CommandTrove {
                 // description
                 Cell::new(&c.description.as_ref().unwrap()[..]),
                 // tags
-                Cell::new(&c.tags.as_ref().unwrap_or(&vec![String::from("")]).join(",")[..]),
+                Cell::new(&c.tags.as_ref().unwrap_or(&vec!["".to_string()]).join(",")[..]),
             ]));
         });
         // Print the table to stdout
@@ -166,4 +184,35 @@ mod test_commands {
         trove.add_command(command);
         assert!(!trove.is_empty());
     }
+
+    #[test]
+    fn trove_namespaces() {
+        let namespace1 = "NAMESPACE1";
+        let namespace2 = "NAMESPACE2";
+
+        let command1 = HoardCommand {
+            name: "name1".to_string(),
+            namespace: namespace1.to_string(),
+            ..HoardCommand::default()
+        };
+
+        let command2 = HoardCommand {
+            name: "name2".to_string(),
+            namespace: namespace2.to_string(),
+            ..HoardCommand::default()
+        };
+
+        let command3 = HoardCommand {
+            name: "name3".to_string(),
+            namespace: namespace1.to_string(),
+            ..HoardCommand::default()
+        };
+
+        let mut trove = CommandTrove::default();
+        trove.add_command(command1);
+        trove.add_command(command2);
+        trove.add_command(command3);
+
+        assert_eq!(vec![namespace1, namespace2], trove.namespaces());
+    }
 }
