@@ -11,11 +11,14 @@ mod command;
 mod config;
 mod gui;
 mod hoard;
+mod util;
 use hoard::Hoard;
 
 fn main() {
     let (command, is_autocomplete) = Hoard::default().with_config(None).load_trove().start();
     if is_autocomplete {
-        eprintln!("{}", command.clone().trim());
+        eprintln!("{}", command.trim());
+    } else {
+        println!("{}", command.trim());
     }
 }
